﻿using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;
using Ivi.Visa.Interop;

//'' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//''  © Agilent Technologies, Inc. 2013
//''
//'' You have a royalty-free right to use, modify, reproduce and distribute
//'' the Sample Application Files (and/or any modified version) in any way
//'' you find useful, provided that you agree that Agilent Technologies has no
//'' warranty,  obligations or liability for any Sample Application Files.
//''
//'' Agilent Technologies provides programming examples for illustration only,
//'' This sample program assumes that you are familiar with the programming
//'' language being demonstrated and the tools used to create and debug
//'' procedures. Agilent Technologies support engineers can help explain the
//'' functionality of Agilent Technologies software components and associated
//'' commands, but they will not modify these samples to provide added
//'' functionality or construct procedures to meet your specific needs.
//'' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

namespace Digitizing
{
    class Digitize
    {
        static void Main(string[] args)
        {
             //This program gives an example of how you can use the 34461A to digitize a signal. It follows closely the settings found for the 34411A Digitizing video
            //found at http://www.youtube.com/watch?v=gxLrF4sgAxk The 34461A does not offer all of the triggering options that the 34411A has so some of the features from
            //the video are not shown in this programming example.

            Digitize DmmClass = new Digitize(); //Create an instance of this class so we can call functions from Main

            //For more information on getting started using VISA COM I/O operations see the app note located at:
            //http://cp.literature.agilent.com/litweb/pdf/5989-6338EN.pdf

            Ivi.Visa.Interop.ResourceManager rm = new Ivi.Visa.Interop.ResourceManager(); //Open up a new resource manager
            Ivi.Visa.Interop.FormattedIO488 myDmm = new Ivi.Visa.Interop.FormattedIO488(); //Open a new Formatted IO 488 session 
            try
            {
                //string DutAddr = "GPIB0::22"; //String for GPIB
                //string DutAddr = ""TCPIP0::169.254.4.61";  //Example string for LAN
                //string DutAddr = "USB0::0x0957::0x1A07::MY53000101"; //Example string for USB
                string DutAddr = "USB0::0x0957::0x1C07::US00000069::0::INSTR"; //String for USB

                myDmm.IO = (IMessage)rm.Open(DutAddr, AccessMode.NO_LOCK, 2000, ""); //Open up a handle to the DMM 
                myDmm.IO.Timeout = 5000;

                //First start with a reset state
                myDmm.IO.Clear(); //Send a device clear first to stop any measurements in process
                myDmm.WriteString("*RST", true); //Reset the device
                myDmm.WriteString("*IDN?", true); //Get the IDN string                
                string IDN = myDmm.ReadString();
                Console.WriteLine(IDN); //report the DMM's identity

                //Configure the 34461 DCV Measurement state
                myDmm.WriteString("CONF:VOLT:DC 10", true);              //Configure DCV, 10V range
                myDmm.WriteString("VOLT:IMP:AUTO OFF", true);       //10M input impedance setting , "ON" sets >10Gohm
                myDmm.WriteString("SENS:VOLT:DC:ZERO:AUTO OFF", true);   // Autozero Off                
                myDmm.WriteString("VOLT:NPLC .02", true);               //Set Integration Time;Resolution to .02 PLC
                DmmClass.CheckDMMError(myDmm);                      //Check the DMM for errors
    
    
                //Configure trigger settings
                myDmm.WriteString("TRIG:SOUR IMM", true);                //Set trigger source to internal
                myDmm.WriteString("TRIG:SLOP POS", true);                //Set trigger slop to pos
                myDmm.WriteString("TRIG:COUN 1", true);                  //Set trigger source to internal                
                myDmm.WriteString("TRIG:DEL:AUTO OFF", true);            //Turn OFF Automatic trigger delay
                myDmm.WriteString("TRIG:DEL 0", true);                   // Set trigger delay to 0 sec
                DmmClass.CheckDMMError(myDmm);                      //Check the DMM for errors
    
                //Configure Sample settings
                myDmm.WriteString("SAMP:COUN 10000", true);              //Set sample count = 10000
                DmmClass.CheckDMMError(myDmm);                      //Check the DMM for errors

                //Trigger and make the readings
                myDmm.IO.Timeout = 15000;                             //Allow 12 seconds for DMM to make readings
                myDmm.WriteString("INIT", true);                           //Put DMM in wait for Trigger state

                myDmm.WriteString("FETCH?", true);                        //Get all the readings from Buffer
    
                //Now Populate the console with the readings
                string Readings = myDmm.ReadString();                 //Get the readings in a string format
                string[] rdgArr = Readings.Split(",".ToCharArray());  //Split the readings into individual elements

                foreach (string rdg in rdgArr) //Write out each result to the console
                    Console.WriteLine(rdg.ToString());


            }
            catch (Exception e)
            {
                Console.WriteLine("Error occured: " + e.Message);
            }
            finally
            {
                //Close out your resources
                try { myDmm.IO.Close(); }
                catch { }
                try { System.Runtime.InteropServices.Marshal.ReleaseComObject(myDmm); }
                catch { }
                try
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(rm);
                }
                catch { }
                Console.WriteLine("Press any key to continue...");
                Console.ReadKey();

            }
        }
            /// <summary>
        /// Checks the error queue and throws an exception if error
        /// </summary>
        /// <param name="myDmm"></param>
        public void CheckDMMError(FormattedIO488 myDmm)
        {

            myDmm.WriteString("SYST:ERR?", true);
            string errStr = myDmm.ReadString();

            if (errStr.Contains("No error")) //If no error, then return
                return;
            //If there is an error, read out all of the errors and return them in an exception
            else
            {
                string errStr2 = "";
                do
                {
                    myDmm.WriteString("SYST:ERR?", true);
                    errStr2 = myDmm.ReadString();
                    if (!errStr2.Contains("No error")) errStr = errStr + "\n" + errStr2;

                } while (!errStr2.Contains("No error"));
                throw new Exception("Exception: Encountered system error(s)\n" + errStr);
            }

        }
    }
}
