﻿using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;
using Ivi.Visa.Interop;
//'' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//''  © Agilent Technologies, Inc. 2013
//''
//'' You have a royalty-free right to use, modify, reproduce and distribute
//'' the Sample Application Files (and/or any modified version) in any way
//'' you find useful, provided that you agree that Agilent Technologies has no
//'' warranty,  obligations or liability for any Sample Application Files.
//''
//'' Agilent Technologies provides programming examples for illustration only,
//'' This sample program assumes that you are familiar with the programming
//'' language being demonstrated and the tools used to create and debug
//'' procedures. Agilent Technologies support engineers can help explain the
//'' functionality of Agilent Technologies software components and associated
//'' commands, but they will not modify these samples to provide added
//'' functionality or construct procedures to meet your specific needs.
//'' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""


namespace MathandStats
{
    class DMMMath
    {
        static void Main(string[] args)
        {
            //Uses the CALC subsystem. This system does internal calculations on the data set include statistics and limit checking. 
            //1)Take 20 readings and turn on internal limits. Set limit and check the questions status byte if a reading was outside of limits.
            //2)Then perform math functions including mean, max, min, standard deviation, and peak to peak. The max and min reading will show the readings that were outside of the limits

            DMMMath DmmClass = new DMMMath(); //Create an instance of this class so we can call functions from Main

            //For more information on getting started using VISA COM I/O operations see the app note located at:
            //http://cp.literature.agilent.com/litweb/pdf/5989-6338EN.pdf

            Ivi.Visa.Interop.ResourceManager rm = new Ivi.Visa.Interop.ResourceManager(); //Open up a new resource manager
            Ivi.Visa.Interop.FormattedIO488 myDmm = new Ivi.Visa.Interop.FormattedIO488(); //Open a new Formatted IO 488 session 
            try
            {
                //string DutAddr = "GPIB0::22"; //String for GPIB
                //string DutAddr = ""TCPIP0::169.254.4.61";  //Example string for LAN
                //string DutAddr = "USB0::0x0957::0x1A07::MY53000101"; //Example string for USB
                string DutAddr = "USB0::0x0957::0x1C07::US00000069::0::INSTR"; //String for GPIB

                myDmm.IO = (IMessage)rm.Open(DutAddr, AccessMode.NO_LOCK, 2000, ""); //Open up a handle to the DMM 
                myDmm.IO.Timeout = 5000;

                //First start with a reset state
                myDmm.IO.Clear(); //Send a device clear first to stop any measurements in process
                myDmm.WriteString("*RST", true); //Reset the device
                myDmm.WriteString("*IDN?", true); //Get the IDN string                
                string IDN = myDmm.ReadString();
                Console.WriteLine(IDN); //report the DMM's identity



                myDmm.WriteString("CONF:VOLT:DC 1, 1e-4", true); //Configure a semi-fast DCV measurement
                myDmm.WriteString("SAMP:COUN 20", true); //Get 20 Readings per trigger

                //Clear all calculations before we start
                myDmm.WriteString("CALC:AVER:CLE", true);
                myDmm.WriteString("CALC:AVER:STAT ON", true); //Turn on Stat calculations for future readings

                //Set the limits and turn on internal limit checking
                myDmm.WriteString("CALC:LIM:LOW -1e-3", true); //Set lower limit to -1mV
                myDmm.WriteString("CALC:LIM:UPP 1e-3", true); //Set upper limit to -1mV

                myDmm.WriteString("CALC:LIM:STAT ON", true); //Turn on Limits for future readings

                //Get the readings
                myDmm.WriteString("READ?", true); //Trigger the DMM to start the readings, can also use INIT command
                Object[] DCVResult = (object[])myDmm.ReadList(IEEEASCIIType.ASCIIType_Any, ",;"); //Get the readings in an array                
                foreach (double rdg in DCVResult) //Write out each result to the console
                    Console.WriteLine(rdg.ToString());

                //Read out the Questionable status register to determine programmatically if there was a limit failure
                myDmm.WriteString("STAT:QUES?", true);
                int Status = (int)myDmm.ReadNumber(IEEEASCIIType.ASCIIType_I4, true);
                //if Bit 12 from status register is set then the upper limit failed
                int uppLimFail = 4096 & Status;
                //If bit 11 from status register is set then the lower limit failed.
                int lowLimFail = 2048 & Status;

                if (uppLimFail == 4096)
                    Console.WriteLine("A reading failed Upper limit!");
                if (lowLimFail == 2048)
                    Console.WriteLine("A reading failed Lower limit!");

                /*Let's get some statistics from the measurements that we just took
                 * 
                 * */
                myDmm.WriteString("CALC:AVER:MAX?", true); //Get maximum from the readings
                double max = (double)myDmm.ReadNumber(IEEEASCIIType.ASCIIType_R4, true);
                DmmClass.CheckDMMError(myDmm); //Check DMM for error
                Console.WriteLine("Max = " + max);
                
                myDmm.WriteString("CALC:AVER:MIN?", true); //Get minimum from the readings
                double min = (double)myDmm.ReadNumber(IEEEASCIIType.ASCIIType_R4, true);
                DmmClass.CheckDMMError(myDmm); //Check DMM for error
                Console.WriteLine("Min = " + min);                                               

                //Let's do some internal math on the readings
                myDmm.WriteString("CALC:AVER:AVER?", true); //Get average from the readings
                double mean = (double)myDmm.ReadNumber(IEEEASCIIType.ASCIIType_R4, true);
                DmmClass.CheckDMMError(myDmm); //Check DMM for error
                Console.WriteLine("Mean = " + mean);

                myDmm.WriteString("CALC:AVER:SDEV?", true); //Get standard dev from the readings
                double sdev = (double)myDmm.ReadNumber(IEEEASCIIType.ASCIIType_R4, true);
                DmmClass.CheckDMMError(myDmm); //Check DMM for error
                Console.WriteLine("Sdev = " + sdev);

                
            }
            catch (Exception e)
            {
                Console.WriteLine("Error occured: " + e.Message);
            }
            finally
            {
                //Close out your resources
                try { myDmm.IO.Close(); }
                catch { }
                try { System.Runtime.InteropServices.Marshal.ReleaseComObject(myDmm); }
                catch { }
                try
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(rm);
                }
                catch { }
                Console.WriteLine("Press any key to continue...");
                Console.ReadKey();

            }

        }
        public void CheckDMMError(FormattedIO488 myDmm)
        {

            myDmm.WriteString("SYST:ERR?", true);
            string errStr = myDmm.ReadString();

            if (errStr.Contains("No error")) //If no error, then return
                return;
            //If there is an error, read out all of the errors and return them in an exception
            else
            {
                string errStr2 = "";
                do
                {
                    myDmm.WriteString("SYST:ERR?", true);
                    errStr2 = myDmm.ReadString();
                    if (!errStr2.Contains("No error")) errStr = errStr + "\n" + errStr2;

                } while (!errStr2.Contains("No error"));
                throw new Exception("Exception: Encountered system error(s)\n" + errStr);
            }

        }
    }
}
